module CarteBlanche_RAM_card
  (
   input ABUS_3M58,
   input ABUS_7M,
   input [15:0] ABUS_A,
   inout [7:0]  ABUS_DB,
   output ABUS_INH,
   input ABUS_PHASE0,
   input ABUS_PHASE1,
   input ABUS_Q3,
   input ABUS_R_W,
   output ABUS_RDY,
   input ABUS_RESET,
   input N_ABUS_DEV_SEL,
   input N_ABUS_IO_SEL,
   
   output [16:0] SRAM_A,
   inout [7:0] SRAM_D,
   output SRAM_NCS0,
   output SRAM_NOE,
   output SRAM_NWE,
   input SYSCLK,
  output LED_BANK1,
  output LED_READ,
  output LED_WRITE,
  output    DUMMY,
  output    DEBUG1

  );

    assign ABUS_RDY = 1'bz;
    assign DUMMY = ABUS_3M58 & ABUS_7M & ABUS_PHASE1 & N_ABUS_DEV_SEL & N_ABUS_IO_SEL;
    
    
    wire [15:0]        addr = ABUS_A;
    reg            bank1, read_en, write_en, pre_wr_en, ph0_1;
   
   wire        clk = SYSCLK;
   wire        reset_in = ~ABUS_RESET;
   wire        addr_is_ROM = addr[15]  & addr[14] & (addr[13] | addr[12]);
   wire        inh_on_board_ROM = addr_is_ROM & 
           ((ABUS_R_W & read_en) | (~ABUS_R_W & write_en));
   

   
   wire        Dxxx;
   wire        strobe = (~ph0_1 & ABUS_PHASE0); // rising edge of PH0
    assign DEBUG1 = strobe;
   
   wire        we = ~ABUS_R_W;
   wire [7:0]      data_to_apple;

   wire [15:0]     ram_addr; // wired to sram
   wire  read_from_card = (addr_is_ROM & read_en) | (addr[15:4] == 'hC08);
   
   assign   SRAM_NCS0 = 0;
   assign SRAM_D = we ? ABUS_DB : 8'bz;
   assign    SRAM_NWE = ~(we & write_en & ABUS_PHASE0 & ~ABUS_Q3);
   assign   SRAM_A = {1'b0, ram_addr};
   
   assign  SRAM_NOE = ~(~we & read_en & ABUS_PHASE0 & ~ABUS_Q3);
   assign data_to_apple = (addr_is_ROM ) ? SRAM_D : {4'h0, pre_wr_en,write_en,read_en,bank1};
   assign ABUS_DB =  (ABUS_R_W & ABUS_PHASE0 & read_from_card )? data_to_apple : 8'bz;

   always @ (posedge clk) begin
      ph0_1 <= ABUS_PHASE0;
   end   

   always @(posedge clk) begin
      if(reset_in) begin
     bank1 <= 0;
     read_en <= 0;
     write_en <= 1;
     pre_wr_en <= 0;
      end else begin
     if((addr[15:4] == 'hC08) & strobe) begin 
        bank1 <= addr[3]; 
        pre_wr_en <= addr[0] & ~we; 
        write_en <= addr[0] & pre_wr_en & ~we; 
        read_en <= ~(addr[0] ^ addr[1]);
     end
      end
   end

   assign ABUS_INH = inh_on_board_ROM ? 1'b0 : 1'bz;
   
   assign Dxxx = (addr[15:12] == 4'b1101);
   assign ram_addr = {addr[15:13], addr[12] & ~(bank1 & Dxxx)  , addr[11:0]};

   assign   LED_BANK1 = bank1;
   assign   LED_READ = read_en;
   assign   LED_WRITE = write_en;


endmodule
/*------------------------------------------------------------*/



